using System.Collections.Concurrent;

namespace ArmaRAMDb
{
    internal class ListStore
    {
        private static readonly ConcurrentDictionary<string, List<string>> _lists = RAMDb._lists;

        public static bool ListAdd(string listName, string value)
        {
            if (_lists.TryGetValue(listName, out var list))
            {
                lock (list)
                {
                    list.Add(value);
                    Main.Log($"ListAdd: {listName} - Value: {value}", "debug");
                    return true;
                }
            }

            Main.Log($"List not found: {listName}", "debug");
            return false;
        }

        public static bool ListDelete(string listName)
        {
            var result = _lists.TryRemove(listName, out _);

            Main.Log($"ListDelete: {listName} - Success: {result}", "debug");
            return result;
        }

        public static string ListIndex(string listName, int index)
        {
            if (_lists.TryGetValue(listName, out var list))
            {
                lock (list)
                {
                    string value = index >= 0 && index < list.Count ? list[index] : string.Empty;
                    Main.Log($"ListIndex: {listName} - Index: {index} - Value: {value}", "debug");
                    return value;
                }
            }

            Main.Log($"List not found: {listName}", "debug");
            return string.Empty;
        }

        public static List<string> ListRange(string listName, int startIndex, int endIndex)
        {            
            if (_lists.TryGetValue(listName, out var list))
            {
               lock (list)
               {
                    startIndex = Math.Max(0, startIndex);
                    endIndex = Math.Min(list.Count - 1, endIndex);

                    if (startIndex <= endIndex)
                    {
                        int count = endIndex - startIndex + 1;
                        var value = list.GetRange(startIndex, count);
                        Main.Log($"ListRange: {listName} - StartIndex: {startIndex} - EndIndex: {endIndex} - Count: {count}", "debug");
                        return value;
                    }
               }
            }
            
            Main.Log($"List not found: {listName}", "debug");
            return [];
        }

        public static bool ListRemove(string listName, string value)
        {
            if (_lists.TryGetValue(listName, out var list))
            {
                lock (list)
                {
                    int removedCount = list.RemoveAll(item => item == value);
                    Main.Log($"ListRemove: {listName} - Value: {value} - Removed Count: {removedCount}", "debug");
                    return removedCount > 0;
                }
            }

            Main.Log($"List not found: {listName}", "debug");
            return false;
        }

        public static bool ListSet(string listName, int index, string value)
        {
            if (_lists.TryGetValue(listName, out var list))
            {
                lock (list)
                {
                    if (index >= 0 && index < list.Count)
                    {
                        list[index] = value;
                        Main.Log($"ListSet: {listName} - Index: {index} - Value: {value}", "debug");
                        return true;
                    }
                }
            }

            Main.Log($"List not found: {listName}", "debug");
            return false;
        }
    }
}